@@ -12,15 +12,55 @@
 #export OS_BUILD=32
 
 #export GCC_TYPE=
-export SFCGAL_VER=1.3.2
-export GEOS_VER=3.7.1
-export GDAL_VER=2.2.4
-export PROJ_VER=4.9.3
-export SFCGAL_VER=1.3.2
+if  [[ "${OVERRIDE}" == '' ]] ; then
+export GEOS_VER=3.8.2
+export GDAL_VER=3.4.2
+export PROJ_VER=7.2.1
+export SFCGAL_VER=1.4.1
 export PCRE_VER=8.33
 export PROTOBUF_VER=3.2.0
 export PROTOBUFC_VER=1.2.1
 export CGAL_VER=4.11
+export BOOST_VER=1.78.0
+	#BOOST_VER_WU=1_49_0
+export BOOST_VER_WU=1_78_0
+
+if [["${OS_BUILD}" == '64']] ; then
+  export GDAL_VER=2.4.4
+  export SFCGAL_VER=1.4.1
+fi;
+
+fi;
+export PROTOBUF_VER=3.2.0
+export PROTOBUFC_VER=1.2.1
+export JSON_VER=0.12
+export PCRE_VER=8.33
+if  [[ "${ICON_VER}" == '' ]] ; then
+  export ICON_VER=1.16
+fi;
+
+echo "ICON_VER ${ICON_VER}"
+
+#set to something even if override is on but not set
+if  [[ "${ZLIB_VER}" == '' ]] ; then
+  export ZLIB_VER=1.2.11
+fi;
+
+if  [[ "${BOOST_VER}" == '' ]] ; then
+  export BOOST_VER=1.78.0
+  export BOOST_VER_WU=1_78_0
+fi;
+
+
+#set to something even if override is on but not set
+if  [[ "${LIBXML_VER}" == '' ]] ; then
+  export LIBXML_VER=2.9.9
+fi;
+
+#set to something even if override is on but not set
+if  [[ "${CGAL_VER}" == '' ]] ; then
+  export CGAL_VER=5.3
+fi;
 
 if [[ "${GCC_TYPE}" == *gcc48* ]] ; then
 	export PROJECTS=/projects
@@ -79,8 +119,7 @@ export REL_PGVER=${PG_VER//./} #strip the period
 export RELDIR=${PROJECTS}/postgis/builds/${POSTGIS_MINOR_VER}
 export RELVERDIR=postgis-pg${REL_PGVER}-binaries-${POSTGIS_MINOR_VER}.${POSTGIS_MICRO_VERSION}w${OS_BUILD}${GCC_TYPE}
 export PATH="${PATHOLD}:${PGPATH}/bin:${PGPATH}/lib"
-export PCRE_VER=8.33 #PATH="${PGPATH}/bin:${PGPATH}/lib:${MINGPROJECTS}/xsltproc:${MINGPROJECTS}/gtkw${OS_BUILD}${GCC_TYPE}/bin:${PROJECTS}/gtkw${OS_BUILD}/bin:${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}${GCC_TYPE}/bin:${MINGPROJECTS}/rel-libiconv-1.13.1w${OS_BUILD}${GCC_TYPE}/include:${MINGPROJECTS}/rel-libiconv-1.13.1w${OS_BUILD}${GCC_TYPE}/bin:${PATH}"
-#echo PATH AFTER: $PATH
+export PCRE_VER=8.33
 outdir="${RELDIR}/${RELVERDIR}"
 package="${RELDIR}/${RELVERDIR}.zip"
 verfile="${RELDIR}/${RELVERDIR}/version.txt"
@@ -97,15 +136,15 @@ mkdir $outdir/bin/postgisgui
 mkdir $outdir/bin/postgisgui/share
 mkdir $outdir/bin/postgisgui/lib
 mkdir $outdir/utils
-cp ${MINGPROJECTS}/rel-libiconv-1.13.1w${OS_BUILD}/bin/*.dll  $outdir/bin/postgisgui
+cp ${MINGPROJECTS}/rel-libiconv-${ICONV_VER}w${OS_BUILD}/bin/*.dll  $outdir/bin/postgisgui
 # it seems 9.2 and 9.3 doesn't come with its own libiconv good grief
 # and trying to use their libiconv2.dll makes shp2pgsql crash
 if [[ "$PG_VER" == *9.2* || "$PG_VER" == *9.3* ]]; then
-	cp ${MINGPROJECTS}/rel-libiconv-1.13.1w${OS_BUILD}${GCC_TYPE}/bin/*.dll  $outdir/bin
+	cp ${MINGPROJECTS}/rel-libiconv-${ICONV_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll  $outdir/bin
 fi;
 cp ${PGPATHEDB}/bin/libpq.dll  $outdir/bin/postgisgui
 #cp ${PGPATHEDB}/bin/libiconv2.dll  $outdir/bin/postgisgui
-cp ${MINGPROJECTS}/rel-libiconv-1.13.1w${OS_BUILD}${GCC_TYPE}/bin/libicon*.dll $outdir/bin/postgisgui
+cp ${MINGPROJECTS}/rel-libiconv-${ICONV_VER}w${OS_BUILD}${GCC_TYPE}/bin/libicon*.dll $outdir/bin/postgisgui
 cp ${PGPATHEDB}/bin/libintl*.dll $outdir/bin/postgisgui
 
 cp ${PGPATHEDB}/bin/ssleay32.dll $outdir/bin/postgisgui
@@ -133,7 +172,7 @@ cp -p ${PROJECTS}/geos/rel-${GEOS_VER}w${OS_BUILD}${GCC_TYPE}/bin/*.dll $outdir/
 #for protobuf
 cp ${PROJECTS}/protobuf/rel-${PROTOBUF_VER}w${OS_BUILD}${GCC_TYPE}/bin/libprotobuf-c-*.dll $outdir/bin
 
-echo "POSTGIS: ${POSTGIS_MINOR_VER} r${POSTGIS_SVN_REVISION} http://postgis.net/source" > $verfile
+echo "POSTGIS: ${POSTGIS_MICRO_VER} https://postgis.net/source" > $verfile
 
 if [ "$POSTGIS_MAJOR_VERSION" == "2" ] ; then
   ## only copy gdal components if 2+.  1.5 doesn't have raster support
@@ -150,8 +189,8 @@ fi;
 
 if [ -n "$SFCGAL_VER"  ]; then
 	## only copy cgal and sfcgal stuff if sfcgal is packaged
-	export BOOST_VER=1.59.0
-	export BOOST_VER_WU=1_59_0
+	export BOOST_VER=1.78.0
+	export BOOST_VER_WU=1_78_0
 	export GMP_VER=5.1.2
 	export MPFR_VER=3.1.2
 	echo "CGAL VERSION: ${CGAL_VER} http://www.cgal.org" >> $verfile
@@ -212,14 +251,12 @@ cp -r extensions/*/*.dll ${RELDIR}/${RELVERDIR}/lib #only address_standardizer i
 cp -r ${RELDIR}/packaging_notes/* ${RELDIR}/${RELVERDIR}/
 
 
-echo "GEOS VERSION: ${GEOS_VER} http://trac.osgeo.org/geos" >> $verfile
-echo "GDAL VERSION: ${GDAL_VER} http://trac.osgeo.org/gdal" >> $verfile
-echo "PROJ VERSION: ${PROJ_VER} http://trac.osgeo.org/proj" >> $verfile
+echo "GEOS VERSION: ${GEOS_VER} https://libgeos.org/usage/download/" >> $verfile
+echo "GDAL VERSION: ${GDAL_VER} https://proj.org/download.html" >> $verfile
+echo "PROJ VERSION: ${PROJ_VER} https://proj.org/download.html" >> $verfile
 
 if [ -n "$SFCGAL_VER"  ]; then
-    echo "CGAL VERSION: ${CGAL_VER} http://www.cgal.org" >> $verfile
-    echo "BOOST VERSION: ${BOOST_VER} http://www.boost.org" >> $verfile
-    echo "SFCGAL VERSION: ${SFCGAL_VER} http://www.sfcgal.org https://github.com/Oslandia/SFCGAL" >> $verfile
+    echo "SFCGAL VERSION: ${SFCGAL_VER} http://www.sfcgal.org https://gitlab.com/Oslandia/SFCGAL" >> $verfile
 fi;
 #echo "PAGC ADDRESS STANDARDIZER: http://sourceforge.net/p/pagc/code/HEAD/tree/branches/sew-refactor/postgresql " >> $verfile
 cd ${RELDIR}
