@@ -157,9 +157,9 @@ UNIT["Meter", 1.0]]'
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'POLYGON' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -176,9 +176,9 @@ UNIT["Meter", 1.0]]'
 -- ---------------------
 CREATE TABLE lakes (
        fid               INTEGER NOT NULL PRIMARY KEY,
-       name              VARCHAR(64)
+       name              VARCHAR(64),
+       shore             GEOMETRY(Polygon, 101)
 );
-SELECT AddGeometryColumn('lakes','shore','101','POLYGON','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -189,9 +189,9 @@ SELECT AddGeometryColumn('lakes','shore','101','POLYGON','2');
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'LINESTRING' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -213,9 +213,9 @@ CREATE TABLE road_segments (
        fid               INTEGER NOT NULL PRIMARY KEY,
        name              VARCHAR(64),
        aliases           VARCHAR(64),
-       num_lanes         INTEGER
+       num_lanes         INTEGER,
+       centerline        GEOMETRY(LineString, 101)
 );
-SELECT AddGeometryColumn('road_segments','centerline','101','LINESTRING','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -228,9 +228,9 @@ SELECT AddGeometryColumn('road_segments','centerline','101','LINESTRING','2');
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'MULTILINESTRING' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -249,9 +249,9 @@ SELECT AddGeometryColumn('road_segments','centerline','101','LINESTRING','2');
 CREATE TABLE divided_routes (
        fid               INTEGER NOT NULL PRIMARY KEY,
        name              VARCHAR(64),
-       num_lanes         INTEGER
+       num_lanes         INTEGER,
+       centerlines       GEOMETRY(multilinestring, 101)
 );
-SELECT AddGeometryColumn('divided_routes','centerlines','101','MULTILINESTRING','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -264,9 +264,9 @@ SELECT AddGeometryColumn('divided_routes','centerlines','101','MULTILINESTRING',
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'MULTIPOLYGON' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -283,9 +283,9 @@ SELECT AddGeometryColumn('divided_routes','centerlines','101','MULTILINESTRING',
 -- ---------------------
 CREATE TABLE forests (
        fid            INTEGER NOT NULL PRIMARY KEY,
-       name           VARCHAR(64)
+       name           VARCHAR(64),
+       boundary       GEOMETRY(MultiPolygon, 101)
 );
-SELECT AddGeometryColumn('forests','boundary','101','MULTIPOLYGON','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -298,9 +298,9 @@ SELECT AddGeometryColumn('forests','boundary','101','MULTIPOLYGON','2');
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'POINT' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -317,9 +317,9 @@ SELECT AddGeometryColumn('forests','boundary','101','MULTIPOLYGON','2');
 -- ---------------------
 CREATE TABLE bridges (
        fid           INTEGER NOT NULL PRIMARY KEY,
-       name          VARCHAR(64)
+       name          VARCHAR(64),
+       position      GEOMETRY(Point, 101)
 );
-SELECT AddGeometryColumn('bridges','position','101','POINT','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -330,9 +330,9 @@ SELECT AddGeometryColumn('bridges','position','101','POINT','2');
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'LINESTRING' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -349,9 +349,9 @@ SELECT AddGeometryColumn('bridges','position','101','POINT','2');
 -- ---------------------
 CREATE TABLE streams (
        fid             INTEGER NOT NULL PRIMARY KEY,
-       name            VARCHAR(64)
+       name            VARCHAR(64),
+       centerline      GEOMETRY(LineString, 101)
 );
-SELECT AddGeometryColumn('streams','centerline','101','LINESTRING','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -369,9 +369,9 @@ SELECT AddGeometryColumn('streams','centerline','101','LINESTRING','2');
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'POINT', 'POLYGON' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -389,10 +389,10 @@ SELECT AddGeometryColumn('streams','centerline','101','LINESTRING','2');
 -- ---------------------
 CREATE TABLE buildings (
        fid             INTEGER NOT NULL PRIMARY KEY,
-       address         VARCHAR(64)
+       address         VARCHAR(64),
+       position        GEOMETRY(Point, 101),
+       footprint       GEOMETRY(Polygon, 101)
 );
-SELECT AddGeometryColumn('buildings','position','101','POINT','2');
-SELECT AddGeometryColumn('buildings','footprint','101','POLYGON','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -405,9 +405,9 @@ SELECT AddGeometryColumn('buildings','footprint','101','POLYGON','2');
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'MULTIPOYLGON' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -426,9 +426,9 @@ SELECT AddGeometryColumn('buildings','footprint','101','POLYGON','2');
 CREATE TABLE ponds (
        fid             INTEGER NOT NULL PRIMARY KEY,
        name            VARCHAR(64),
-       type            VARCHAR(64)
+       type            VARCHAR(64),
+       shores          GEOMETRY(MultiPolygon, 101)
 );
-SELECT AddGeometryColumn('ponds','shores','101','MULTIPOLYGON','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -441,9 +441,9 @@ SELECT AddGeometryColumn('ponds','shores','101','MULTIPOLYGON','2');
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'POLYGON' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -461,9 +461,9 @@ SELECT AddGeometryColumn('ponds','shores','101','MULTIPOLYGON','2');
 -- ---------------------
 CREATE TABLE named_places (
        fid             INTEGER NOT NULL PRIMARY KEY,
-       name            VARCHAR(64)
+       name            VARCHAR(64),
+       boundary        GEOMETRY(Polygon, 101)
 );
-SELECT AddGeometryColumn('named_places','boundary','101','POLYGON','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -475,9 +475,9 @@ SELECT AddGeometryColumn('named_places','boundary','101','POLYGON','2');
 --
 --
 -- !#@ ADAPTATION BEGIN
--- We break the schema creation into two steps, first create the
--- attribute table, second use the AddGeometryColumn() function
--- to create and register the geometry column.
+-- We declare the geometry column type using 'GEOMETRY'
+-- as the type with 'POLYGON' and 101 as type modifiers
+-- to enforce the geometry type and SRID.
 -- ---------------------
 -- -- BEGIN ORIGINAL SQL
 -- ---------------------
@@ -492,9 +492,9 @@ SELECT AddGeometryColumn('named_places','boundary','101','POLYGON','2');
 -- -- BEGIN ADAPTED  SQL
 -- ---------------------
 CREATE TABLE map_neatlines (
-       fid             INTEGER NOT NULL PRIMARY KEY
+       fid             INTEGER NOT NULL PRIMARY KEY,
+       neatline        GEOMETRY(Polygon, 101)
 );
-SELECT AddGeometryColumn('map_neatlines','neatline','101','POLYGON','2');
 -- ---------------------
 -- -- END   ADAPTED  SQL
 -- ---------------------
@@ -527,7 +527,7 @@ SELECT AddGeometryColumn('map_neatlines','neatline','101','POLYGON','2');
 --
 --
 INSERT INTO lakes VALUES (101, 'Blue Lake',
-    PolygonFromText('POLYGON((52 18,66 23,73 9,48 6,52 18),(59 18,67 18,67 13,59 13,59 18))', 101)
+    ST_GeomFromText('POLYGON((52 18,66 23,73 9,48 6,52 18),(59 18,67 18,67 13,59 13,59 18))', 101)
 );
 --
 --==================
@@ -550,19 +550,19 @@ INSERT INTO lakes VALUES (101, 'Blue Lake',
 --
 --
 INSERT INTO road_segments VALUES(102, 'Route 5', NULL, 2,
-    LineStringFromText('LINESTRING( 0 18, 10 21, 16 23, 28 26, 44 31 )' ,101)
+    ST_GeomFromText('LINESTRING( 0 18, 10 21, 16 23, 28 26, 44 31 )' ,101)
 );
 INSERT INTO road_segments VALUES(103, 'Route 5', 'Main Street', 4,
-    LineStringFromText('LINESTRING( 44 31, 56 34, 70 38 )' ,101)
+    ST_GeomFromText('LINESTRING( 44 31, 56 34, 70 38 )' ,101)
 );
 INSERT INTO road_segments VALUES(104, 'Route 5', NULL, 2,
-    LineStringFromText('LINESTRING( 70 38, 72 48 )' ,101)
+    ST_GeomFromText('LINESTRING( 70 38, 72 48 )' ,101)
 );
 INSERT INTO road_segments VALUES(105, 'Main Street', NULL, 4,
-    LineStringFromText('LINESTRING( 70 38, 84 42 )' ,101)
+    ST_GeomFromText('LINESTRING( 70 38, 84 42 )' ,101)
 );
 INSERT INTO road_segments VALUES(106, 'Dirt Road by Green Forest', NULL, 1,
-    LineStringFromText('LINESTRING( 28 26, 28 0 )',101)
+    ST_GeomFromText('LINESTRING( 28 26, 28 0 )',101)
 );
 --
 --==================
@@ -575,7 +575,7 @@ INSERT INTO road_segments VALUES(106, 'Dirt Road by Green Forest', NULL, 1,
 --==================
 --
 INSERT INTO divided_routes VALUES(119, 'Route 75', 4,
-    MultiLineStringFromText('MULTILINESTRING((10 48,10 21,10 0),(16 0,16 23,16 48))', 101)
+    ST_GeomFromText('MULTILINESTRING((10 48,10 21,10 0),(16 0,16 23,16 48))', 101)
 );
 --
 --==================
@@ -590,7 +590,7 @@ INSERT INTO divided_routes VALUES(119, 'Route 75', 4,
 --==================
 --
 INSERT INTO forests VALUES(109, 'Green Forest',
-    MultiPolygonFromText('MULTIPOLYGON(((28 26,28 0,84 0,84 42,28 26),(52 18,66 23,73 9,48 6,52 18)),((59 18,67 18,67 13,59 13,59 18)))', 101)
+    ST_GeomFromText('MULTIPOLYGON(((28 26,28 0,84 0,84 42,28 26),(52 18,66 23,73 9,48 6,52 18)),((59 18,67 18,67 13,59 13,59 18)))', 101)
 );
 --
 --==================
@@ -603,7 +603,7 @@ INSERT INTO forests VALUES(109, 'Green Forest',
 --==================
 --
 INSERT INTO bridges VALUES(110, 'Cam Bridge',
-    PointFromText('POINT( 44 31 )', 101)
+    ST_GeomFromText('POINT( 44 31 )', 101)
 );
 --
 --==================
@@ -617,10 +617,10 @@ INSERT INTO bridges VALUES(110, 'Cam Bridge',
 --==================
 --
 INSERT INTO streams VALUES(111, 'Cam Stream',
-    LineStringFromText('LINESTRING( 38 48, 44 41, 41 36, 44 31, 52 18 )', 101)
+    ST_GeomFromText('LINESTRING( 38 48, 44 41, 41 36, 44 31, 52 18 )', 101)
 );
 INSERT INTO streams VALUES(112, NULL,
-    LineStringFromText('LINESTRING( 76 0, 78 4, 73 9 )', 101)
+    ST_GeomFromText('LINESTRING( 76 0, 78 4, 73 9 )', 101)
 );
 --
 --==================
@@ -638,12 +638,12 @@ INSERT INTO streams VALUES(112, NULL,
 --==================
 --
 INSERT INTO buildings VALUES(113, '123 Main Street',
-    PointFromText('POINT( 52 30 )', 101),
-    PolygonFromText('POLYGON( ( 50 31, 54 31, 54 29, 50 29, 50 31) )', 101)
+    ST_GeomFromText('POINT( 52 30 )', 101),
+    ST_GeomFromText('POLYGON( ( 50 31, 54 31, 54 29, 50 29, 50 31) )', 101)
 );
 INSERT INTO buildings VALUES(114, '215 Main Street',
-    PointFromText('POINT( 64 33 )', 101),
-    PolygonFromText('POLYGON( ( 66 34, 62 34, 62 32, 66 32, 66 34) )', 101)
+    ST_GeomFromText('POINT( 64 33 )', 101),
+    ST_GeomFromText('POLYGON( ( 66 34, 62 34, 62 32, 66 32, 66 34) )', 101)
 );
 --
 --==================
@@ -656,7 +656,7 @@ INSERT INTO buildings VALUES(114, '215 Main Street',
 --==================
 --
 INSERT INTO ponds VALUES(120, NULL, 'Stock Pond',
-    MultiPolygonFromText('MULTIPOLYGON( ( ( 24 44, 22 42, 24 40, 24 44) ), ( ( 26 44, 26 40, 28 42, 26 44) ) )', 101)
+    ST_GeomFromText('MULTIPOLYGON( ( ( 24 44, 22 42, 24 40, 24 44) ), ( ( 26 44, 26 40, 28 42, 26 44) ) )', 101)
 );
 --
 --==================
@@ -672,10 +672,10 @@ INSERT INTO ponds VALUES(120, NULL, 'Stock Pond',
 --==================
 --
 INSERT INTO named_places VALUES(117, 'Ashton',
-    PolygonFromText('POLYGON( ( 62 48, 84 48, 84 30, 56 30, 56 34, 62 48) )', 101)
+    ST_GeomFromText('POLYGON( ( 62 48, 84 48, 84 30, 56 30, 56 34, 62 48) )', 101)
 );
 INSERT INTO named_places VALUES(118, 'Goose Island',
-    PolygonFromText('POLYGON( ( 67 13, 67 18, 59 18, 59 13, 67 13) )', 101)
+    ST_GeomFromText('POLYGON( ( 67 13, 67 18, 59 18, 59 13, 67 13) )', 101)
 );
 --
 --==================
@@ -688,7 +688,7 @@ INSERT INTO named_places VALUES(118, 'Goose Island',
 --==================
 --
 INSERT INTO map_neatlines VALUES(115,
-    PolygonFromText('POLYGON( ( 0 0, 0 48, 84 48, 84 0, 0 0 ) )', 101)
+    ST_GeomFromText('POLYGON( ( 0 0, 0 48, 84 48, 84 0, 0 0 ) )', 101)
 );
 --
 --
