@@ -16,6 +16,7 @@
 #include "cu_tester.h"
 #include "../postgis_config.h"
 
+char cu_error_msg[MAX_CUNIT_ERROR_LENGTH + 1] = {0};
 /* Internal funcs */
 static void
 cu_errorreporter(const char *fmt, va_list ap);
@@ -47,6 +48,8 @@ extern void libgeom_suite_setup(void);
 extern void lwstroke_suite_setup(void);
 extern void measures_suite_setup(void);
 extern void effectivearea_suite_setup(void);
+extern void chaikin_suite_setup(void);
+extern void filterm_suite_setup(void);
 extern void minimum_bounding_circle_suite_setup(void);
 extern void misc_suite_setup(void);
 extern void node_suite_setup(void);
@@ -97,6 +100,8 @@ PG_SuiteSetup setupfuncs[] =
 	lwstroke_suite_setup,
 	measures_suite_setup,
 	effectivearea_suite_setup,
+	chaikin_suite_setup,
+	filterm_suite_setup,
 	minimum_bounding_circle_suite_setup,
 	misc_suite_setup,
 	node_suite_setup,
@@ -140,7 +145,7 @@ int main(int argc, char *argv[])
 	char *suite_name;
 	CU_pSuite suite_to_run;
 	char *test_name;
-	CU_pTest test_to_run;
+	CU_pTest test_to_run = NULL;
 	CU_ErrorCode errCode = 0;
 	CU_pTestRegistry registry;
 	int num_run;
@@ -206,7 +211,7 @@ int main(int argc, char *argv[])
 			}
 			else
 			{
-				if (test_name != NULL)
+				if (test_name != NULL && test_to_run != NULL)
 				{
 					/* Run only this test. */
 					printf("\nRunning test '%s' in suite '%s'.\n", test_name, suite_name);
