@@ -171,7 +171,7 @@ lwcollection_clone_deep(const LWCOLLECTION *g)
 /**
  * Ensure the collection can hold up at least ngeoms
  */
-void lwcollection_reserve(LWCOLLECTION *col, int ngeoms)
+void lwcollection_reserve(LWCOLLECTION *col, uint32_t ngeoms)
 {
 	if ( ngeoms <= col->maxgeoms ) return;
 
@@ -186,9 +186,10 @@ void lwcollection_reserve(LWCOLLECTION *col, int ngeoms)
 */
 LWCOLLECTION* lwcollection_add_lwgeom(LWCOLLECTION *col, const LWGEOM *geom)
 {
-	if ( col == NULL || geom == NULL ) return NULL;
+	if (!col || !geom) return NULL;
 
-	if ( col->geoms == NULL && (col->ngeoms || col->maxgeoms) ) {
+	if (!col->geoms && (col->ngeoms || col->maxgeoms))
+	{
 		lwerror("Collection is in inconsistent state. Null memory but non-zero collection counts.");
 		return NULL;
 	}
@@ -200,7 +201,7 @@ LWCOLLECTION* lwcollection_add_lwgeom(LWCOLLECTION *col, const LWGEOM *geom)
 	}
 
 	/* In case this is a truly empty, make some initial space  */
-	if ( col->geoms == NULL )
+	if (!col->geoms)
 	{
 		col->maxgeoms = 2;
 		col->ngeoms = 0;
@@ -214,17 +215,16 @@ LWCOLLECTION* lwcollection_add_lwgeom(LWCOLLECTION *col, const LWGEOM *geom)
 	/* See http://trac.osgeo.org/postgis/ticket/2933 */
 	/* Make sure we don't already have a reference to this geom */
 	{
-	int i = 0;
-	for ( i = 0; i < col->ngeoms; i++ )
-	{
-		if ( col->geoms[i] == geom )
+		uint32_t i = 0;
+		for (i = 0; i < col->ngeoms; i++)
 		{
-            lwerror("%s [%d] found duplicate geometry in collection %p == %p", __FILE__, __LINE__, col->geoms[i], geom);
-			LWDEBUGF(4, "Found duplicate geometry in collection %p == %p", col->geoms[i], geom);
-			return col;
+			if (col->geoms[i] == geom)
+			{
+				lwerror("%s [%d] found duplicate geometry in collection %p == %p", __FILE__, __LINE__, col->geoms[i], geom);
+				return col;
+			}
 		}
 	}
-	}
 #endif
 
 	col->geoms[col->ngeoms] = (LWGEOM*)geom;
@@ -232,27 +232,43 @@ LWCOLLECTION* lwcollection_add_lwgeom(LWCOLLECTION *col, const LWGEOM *geom)
 	return col;
 }
 
-
+/**
+ * Appends all geometries from col2 to col1 in place.
+ * Caller is responsible to release col2.
+ */
 LWCOLLECTION *
-lwcollection_segmentize2d(LWCOLLECTION *col, double dist)
+lwcollection_concat_in_place(LWCOLLECTION *col1, const LWCOLLECTION *col2)
 {
 	uint32_t i;
-	LWGEOM **newgeoms;
+	if (!col1 || !col2) return NULL;
+	for (i = 0; i < col2->ngeoms; i++)
+		col1 = lwcollection_add_lwgeom(col1, col2->geoms[i]);
+	return col1;
+}
 
-	if ( ! col->ngeoms ) return lwcollection_clone(col);
+LWCOLLECTION*
+lwcollection_segmentize2d(const LWCOLLECTION* col, double dist)
+{
+	uint32_t i, j;
+	LWGEOM** newgeoms;
 
-	newgeoms = lwalloc(sizeof(LWGEOM *)*col->ngeoms);
-	for (i=0; i<col->ngeoms; i++)
+	if (!col->ngeoms) return lwcollection_clone(col);
+
+	newgeoms = lwalloc(sizeof(LWGEOM*) * col->ngeoms);
+	for (i = 0; i < col->ngeoms; i++)
 	{
 		newgeoms[i] = lwgeom_segmentize2d(col->geoms[i], dist);
-		if ( ! newgeoms[i] ) {
-			while (i--) lwgeom_free(newgeoms[i]);
+		if (!newgeoms[i])
+		{
+			for (j = 0; j < i; j++)
+				lwgeom_free(newgeoms[j]);
 			lwfree(newgeoms);
 			return NULL;
 		}
 	}
 
-	return lwcollection_construct(col->type, col->srid, NULL, col->ngeoms, newgeoms);
+	return lwcollection_construct(
+	    col->type, col->srid, NULL, col->ngeoms, newgeoms);
 }
 
 /** @brief check for same geometry composition
@@ -301,7 +317,7 @@ lwcollection_same(const LWCOLLECTION *c1, const LWCOLLECTION *c2)
 
 int lwcollection_ngeoms(const LWCOLLECTION *col)
 {
-	int i;
+	uint32_t i;
 	int ngeoms = 0;
 
 	if ( ! col )
@@ -339,7 +355,7 @@ int lwcollection_ngeoms(const LWCOLLECTION *col)
 
 void lwcollection_free(LWCOLLECTION *col)
 {
-	int i;
+	uint32_t i;
 	if ( ! col ) return;
 
 	if ( col->bbox )
@@ -366,16 +382,17 @@ void lwcollection_free(LWCOLLECTION *col)
 * WARNING: the output will contain references to geometries in the input,
 * so the result must be carefully released, not freed.
 */
-LWCOLLECTION* lwcollection_extract(LWCOLLECTION *col, int type)
+LWCOLLECTION*
+lwcollection_extract(LWCOLLECTION* col, int type)
 {
-	int i = 0;
-	LWGEOM **geomlist;
-	LWCOLLECTION *outcol;
+	uint32_t i = 0;
+	LWGEOM** geomlist;
+	LWCOLLECTION* outcol;
 	int geomlistsize = 16;
 	int geomlistlen = 0;
 	uint8_t outtype;
 
-	if ( ! col ) return NULL;
+	if (!col) return NULL;
 
 	switch (type)
 	{
@@ -389,91 +406,81 @@ LWCOLLECTION* lwcollection_extract(LWCOLLECTION *col, int type)
 		outtype = MULTIPOLYGONTYPE;
 		break;
 	default:
-		lwerror("Only POLYGON, LINESTRING and POINT are supported by lwcollection_extract. %s requested.", lwtype_name(type));
+		lwerror(
+		    "Only POLYGON, LINESTRING and POINT are supported by "
+		    "lwcollection_extract. %s requested.",
+		    lwtype_name(type));
 		return NULL;
 	}
 
 	geomlist = lwalloc(sizeof(LWGEOM*) * geomlistsize);
 
 	/* Process each sub-geometry */
-	for ( i = 0; i < col->ngeoms; i++ )
+	for (i = 0; i < col->ngeoms; i++)
 	{
 		int subtype = col->geoms[i]->type;
 		/* Don't bother adding empty sub-geometries */
-		if ( lwgeom_is_empty(col->geoms[i]) )
-		{
-			continue;
-		}
+		if (lwgeom_is_empty(col->geoms[i])) continue;
 		/* Copy our sub-types into the output list */
-		if ( subtype == type )
+		if (subtype == type)
 		{
-			/* We've over-run our buffer, double the memory segment */
-			if ( geomlistlen == geomlistsize )
+			/* We've over-run our buffer, double the memory segment
+			 */
+			if (geomlistlen == geomlistsize)
 			{
 				geomlistsize *= 2;
-				geomlist = lwrealloc(geomlist, sizeof(LWGEOM*) * geomlistsize);
+				geomlist = lwrealloc(
+				    geomlist, sizeof(LWGEOM*) * geomlistsize);
 			}
 			geomlist[geomlistlen] = lwgeom_clone(col->geoms[i]);
 			geomlistlen++;
 		}
 		/* Recurse into sub-collections */
-		if ( lwtype_is_collection( subtype ) )
+		if (lwtype_is_collection(subtype))
 		{
-			int j = 0;
-			LWCOLLECTION *tmpcol = lwcollection_extract((LWCOLLECTION*)col->geoms[i], type);
-			for ( j = 0; j < tmpcol->ngeoms; j++ )
+			uint32_t j = 0;
+			LWCOLLECTION* tmpcol = lwcollection_extract(
+			    (LWCOLLECTION*)col->geoms[i], type);
+			for (j = 0; j < tmpcol->ngeoms; j++)
 			{
-				/* We've over-run our buffer, double the memory segment */
-				if ( geomlistlen == geomlistsize )
+				/* We've over-run our buffer, double the memory
+				 * segment */
+				if (geomlistlen == geomlistsize)
 				{
 					geomlistsize *= 2;
-					geomlist = lwrealloc(geomlist, sizeof(LWGEOM*) * geomlistsize);
+					geomlist = lwrealloc(geomlist,
+							     sizeof(LWGEOM*) *
+								 geomlistsize);
 				}
 				geomlist[geomlistlen] = tmpcol->geoms[j];
 				geomlistlen++;
 			}
-			if (tmpcol->geoms)
-				lwfree(tmpcol->geoms);
-			if (tmpcol->bbox)
-				lwfree(tmpcol->bbox);
+			if (tmpcol->ngeoms) lwfree(tmpcol->geoms);
+			if (tmpcol->bbox) lwfree(tmpcol->bbox);
 			lwfree(tmpcol);
 		}
 	}
 
-	if ( geomlistlen > 0 )
+	if (geomlistlen > 0)
 	{
 		GBOX gbox;
-		outcol = lwcollection_construct(outtype, col->srid, NULL, geomlistlen, geomlist);
-		lwgeom_calculate_gbox((LWGEOM *) outcol, &gbox);
+		outcol = lwcollection_construct(
+		    outtype, col->srid, NULL, geomlistlen, geomlist);
+		lwgeom_calculate_gbox((LWGEOM*)outcol, &gbox);
 		outcol->bbox = gbox_copy(&gbox);
 	}
 	else
 	{
 		lwfree(geomlist);
-		outcol = lwcollection_construct_empty(outtype, col->srid, FLAGS_GET_Z(col->flags), FLAGS_GET_M(col->flags));
+		outcol = lwcollection_construct_empty(outtype,
+						      col->srid,
+						      FLAGS_GET_Z(col->flags),
+						      FLAGS_GET_M(col->flags));
 	}
 
 	return outcol;
 }
 
-LWGEOM*
-lwcollection_remove_repeated_points(const LWCOLLECTION *coll, double tolerance)
-{
-	uint32_t i;
-	LWGEOM **newgeoms;
-
-	newgeoms = lwalloc(sizeof(LWGEOM *)*coll->ngeoms);
-	for (i=0; i<coll->ngeoms; i++)
-	{
-		newgeoms[i] = lwgeom_remove_repeated_points(coll->geoms[i], tolerance);
-	}
-
-	return (LWGEOM*)lwcollection_construct(coll->type,
-	                                       coll->srid, coll->bbox ? gbox_copy(coll->bbox) : NULL,
-	                                       coll->ngeoms, newgeoms);
-}
-
-
 LWCOLLECTION*
 lwcollection_force_dims(const LWCOLLECTION *col, int hasz, int hasm)
 {
@@ -486,7 +493,7 @@ lwcollection_force_dims(const LWCOLLECTION *col, int hasz, int hasm)
 	}
 	else
 	{
-		int i;
+		uint32_t i;
 		LWGEOM **geoms = NULL;
 		geoms = lwalloc(sizeof(LWGEOM*) * col->ngeoms);
 		for( i = 0; i < col->ngeoms; i++ )
@@ -500,7 +507,7 @@ lwcollection_force_dims(const LWCOLLECTION *col, int hasz, int hasm)
 
 int lwcollection_is_empty(const LWCOLLECTION *col)
 {
-	int i;
+	uint32_t i;
 	if ( (col->ngeoms == 0) || (!col->geoms) )
 		return LW_TRUE;
 	for( i = 0; i < col->ngeoms; i++ )
@@ -511,10 +518,10 @@ int lwcollection_is_empty(const LWCOLLECTION *col)
 }
 
 
-int lwcollection_count_vertices(LWCOLLECTION *col)
+uint32_t lwcollection_count_vertices(LWCOLLECTION *col)
 {
-	int i = 0;
-	int v = 0; /* vertices */
+	uint32_t i = 0;
+	uint32_t v = 0; /* vertices */
 	assert(col);
 	for ( i = 0; i < col->ngeoms; i++ )
 	{
@@ -523,22 +530,6 @@ int lwcollection_count_vertices(LWCOLLECTION *col)
 	return v;
 }
 
-LWCOLLECTION* lwcollection_simplify(const LWCOLLECTION *igeom, double dist, int preserve_collapsed)
-{
- 	int i;
-	LWCOLLECTION *out = lwcollection_construct_empty(igeom->type, igeom->srid, FLAGS_GET_Z(igeom->flags), FLAGS_GET_M(igeom->flags));
-
-	if( lwcollection_is_empty(igeom) )
-		return out; /* should we return NULL instead ? */
-
-	for( i = 0; i < igeom->ngeoms; i++ )
-	{
-		LWGEOM *ngeom = lwgeom_simplify(igeom->geoms[i], dist, preserve_collapsed);
-		if ( ngeom ) out = lwcollection_add_lwgeom(out, ngeom);
-	}
-
-	return out;
-}
 
 int lwcollection_allows_subtype(int collectiontype, int subtype)
 {
@@ -586,19 +577,3 @@ lwcollection_startpoint(const LWCOLLECTION* col, POINT4D* pt)
 }
 
 
-LWCOLLECTION* lwcollection_grid(const LWCOLLECTION *coll, const gridspec *grid)
-{
-	uint32_t i;
-	LWCOLLECTION *newcoll;
-
-	newcoll = lwcollection_construct_empty(coll->type, coll->srid, lwgeom_has_z((LWGEOM*)coll), lwgeom_has_m((LWGEOM*)coll));
-
-	for (i=0; i<coll->ngeoms; i++)
-	{
-		LWGEOM *g = lwgeom_grid(coll->geoms[i], grid);
-		if ( g )
-			lwcollection_add_lwgeom(newcoll, g);
-	}
-
-	return newcoll;
-}
