@@ -133,19 +133,19 @@ GIDX* gidx_copy(GIDX *b)
 
 
 /* Ensure all minimums are below maximums. */
-static inline void gidx_validate(GIDX *b)
+void
+gidx_validate(GIDX *b)
 {
 	uint32_t i;
 	Assert(b);
-	POSTGIS_DEBUGF(5,"validating gidx (%s)", gidx_to_string(b));
-	for ( i = 0; i < GIDX_NDIMS(b); i++ )
+	POSTGIS_DEBUGF(5, "validating gidx (%s)", gidx_to_string(b));
+	for (i = 0; i < GIDX_NDIMS(b); i++)
 	{
-		if ( GIDX_GET_MIN(b,i) > GIDX_GET_MAX(b,i) )
+		if (GIDX_GET_MIN(b, i) > GIDX_GET_MAX(b, i))
 		{
-			float tmp;
-			tmp = GIDX_GET_MIN(b,i);
-			GIDX_SET_MIN(b,i,GIDX_GET_MAX(b,i));
-			GIDX_SET_MAX(b,i,tmp);
+			float tmp = GIDX_GET_MIN(b, i);
+			GIDX_SET_MIN(b, i, GIDX_GET_MAX(b, i));
+			GIDX_SET_MAX(b, i, tmp);
 		}
 	}
 	return;
@@ -184,7 +184,8 @@ void gidx_merge(GIDX **b_union, GIDX *b_new)
 	/* Merge of unknown and known is known */
 	if( gidx_is_unknown(*b_union) )
 	{
-		*b_union = b_new;
+		pfree(*b_union);
+		*b_union = gidx_copy(b_new);
 		return;
 	}
 
@@ -856,7 +857,7 @@ Datum gserialized_distance_nd(PG_FUNCTION_ARGS)
 	/* Can only add the M term if both objects have M */
 	if ( lwgeom_has_m(lw1) && lwgeom_has_m(lw2) )
 	{
-		double m1, m2;
+		double m1 = 0, m2 = 0;
 		int usebox = false;
 
 		if ( lwgeom_get_type(lw1) == POINTTYPE )
