@@ -73,7 +73,6 @@
 #include <postgres.h>
 #include <utils/builtins.h>    /* For float manipulation */
 #include "access/spgist.h"     /* For SP-GiST */
-#include "catalog/namespace.h" /* For TypenameGetTypid */
 #include "catalog/pg_type_d.h" /* For VOIDOID */
 
 #include "../postgis_config.h"
@@ -290,8 +289,10 @@ PG_FUNCTION_INFO_V1(gserialized_spgist_config_2d);
 PGDLLEXPORT Datum gserialized_spgist_config_2d(PG_FUNCTION_ARGS)
 {
 	spgConfigOut *cfg = (spgConfigOut *)PG_GETARG_POINTER(1);
+	Oid boxoid = InvalidOid;
+	postgis_initialize_cache(fcinfo);
+	boxoid = postgis_oid(BOX2DFOID);
 
-	Oid boxoid = TypenameGetTypid("box2df");
 	cfg->prefixType = boxoid;
 	cfg->labelType = VOIDOID; /* We don't need node labels. */
 	cfg->leafType = boxoid;
