@@ -72,8 +72,8 @@ Datum LWGEOM_simplify2d(PG_FUNCTION_ARGS)
 	bool preserve_collapsed = false;
 
 	/* Handle optional argument to preserve collapsed features */
-	if ( PG_NARGS() > 2 && ! PG_ARGISNULL(2) )
-		preserve_collapsed = true;
+	if ((PG_NARGS() > 2) && (!PG_ARGISNULL(2)))
+		preserve_collapsed = PG_GETARG_BOOL(2);
 
 	/* Can't simplify points! */
 	if ( type == POINTTYPE || type == MULTIPOINTTYPE )
@@ -85,7 +85,8 @@ Datum LWGEOM_simplify2d(PG_FUNCTION_ARGS)
 	if ( ! out ) PG_RETURN_NULL();
 
 	/* COMPUTE_BBOX TAINTING */
-	if ( in->bbox ) lwgeom_add_bbox(out);
+	if (in->bbox)
+		lwgeom_refresh_bbox(out);
 
 	result = geometry_serialize(out);
 	lwgeom_free(out);
@@ -330,8 +331,7 @@ Datum LWGEOM_snaptogrid(PG_FUNCTION_ARGS)
 
 	/* COMPUTE_BBOX TAINTING */
 	if ( in_lwgeom->bbox )
-		lwgeom_add_bbox(out_lwgeom);
-
+		lwgeom_refresh_bbox(out_lwgeom);
 
 	POSTGIS_DEBUGF(3, "SnapToGrid made a %s", lwtype_name(out_lwgeom->type));
 
@@ -412,7 +412,10 @@ Datum LWGEOM_snaptogrid_pointoff(PG_FUNCTION_ARGS)
 	if ( out_lwgeom == NULL ) PG_RETURN_NULL();
 
 	/* COMPUTE_BBOX TAINTING */
-	if ( in_lwgeom->bbox ) lwgeom_add_bbox(out_lwgeom);
+	if (in_lwgeom->bbox)
+	{
+		lwgeom_refresh_bbox(out_lwgeom);
+	}
 
 	POSTGIS_DEBUGF(3, "SnapToGrid made a %s", lwtype_name(out_lwgeom->type));
 
@@ -740,7 +743,7 @@ static int point_in_ring_rtree(RTREE_NODE *root, const POINT2D *point)
 		 * then the line is to the right of the point and
 		 * circling counter-clockwise, so increment.
 		 */
-		if (FP_CONTAINS_BOTTOM(seg1->y, point->y, seg2->y) && side>0)
+		if ((seg1->y <= point->y) && (point->y < seg2->y) && (side > 0))
 		{
 			POSTGIS_DEBUG(3, "incrementing winding number.");
 
@@ -751,7 +754,7 @@ static int point_in_ring_rtree(RTREE_NODE *root, const POINT2D *point)
 		 * then the line is to the right of the point and circling
 		 * clockwise, so decrement.
 		 */
-		else if (FP_CONTAINS_BOTTOM(seg2->y, point->y, seg1->y) && side<0)
+		else if ((seg2->y <= point->y) && (point->y < seg1->y) && (side < 0))
 		{
 			POSTGIS_DEBUG(3, "decrementing winding number.");
 
@@ -795,7 +798,7 @@ static int point_in_ring(POINTARRAY *pts, const POINT2D *point)
 		POSTGIS_DEBUGF(3, "counterclockwise wrap %d, clockwise wrap %d", FP_CONTAINS_BOTTOM(seg1->y, point->y, seg2->y), FP_CONTAINS_BOTTOM(seg2->y, point->y, seg1->y));
 
 		/* zero length segments are ignored. */
-		if (((seg2->x - seg1->x)*(seg2->x - seg1->x) + (seg2->y - seg1->y)*(seg2->y - seg1->y)) < 1e-12*1e-12)
+		if ((seg2->x == seg1->x) && (seg2->y == seg1->y))
 		{
 			POSTGIS_DEBUG(3, "segment is zero length... ignoring.");
 
@@ -819,7 +822,7 @@ static int point_in_ring(POINTARRAY *pts, const POINT2D *point)
 		 * then the line is to the right of the point and
 		 * circling counter-clockwise, so increment.
 		 */
-		if (FP_CONTAINS_BOTTOM(seg1->y, point->y, seg2->y) && side>0)
+		if ((seg1->y <= point->y) && (point->y < seg2->y) && (side > 0))
 		{
 			POSTGIS_DEBUG(3, "incrementing winding number.");
 
@@ -830,7 +833,7 @@ static int point_in_ring(POINTARRAY *pts, const POINT2D *point)
 		 * then the line is to the right of the point and circling
 		 * clockwise, so decrement.
 		 */
-		else if (FP_CONTAINS_BOTTOM(seg2->y, point->y, seg1->y) && side<0)
+		else if ((seg2->y <= point->y) && (point->y < seg1->y) && (side < 0))
 		{
 			POSTGIS_DEBUG(3, "decrementing winding number.");
 
