@@ -2166,21 +2166,20 @@ Datum LWGEOM_addpoint(PG_FUNCTION_ARGS)
 	GSERIALIZED *pglwg1, *pglwg2, *result;
 	LWPOINT *point;
 	LWLINE *line, *linecopy;
-	int32 where = -1;
+	uint32_t uwhere = 0;
 
 	POSTGIS_DEBUGF(2, "%s called.", __func__);
 
 	pglwg1 = PG_GETARG_GSERIALIZED_P(0);
 	pglwg2 = PG_GETARG_GSERIALIZED_P(1);
 
-
-	if ( gserialized_get_type(pglwg1) != LINETYPE )
+	if (gserialized_get_type(pglwg1) != LINETYPE)
 	{
 		elog(ERROR, "First argument must be a LINESTRING");
 		PG_RETURN_NULL();
 	}
 
-	if ( gserialized_get_type(pglwg2) != POINTTYPE )
+	if (gserialized_get_type(pglwg2) != POINTTYPE)
 	{
 		elog(ERROR, "Second argument must be a POINT");
 		PG_RETURN_NULL();
@@ -2188,26 +2187,33 @@ Datum LWGEOM_addpoint(PG_FUNCTION_ARGS)
 
 	line = lwgeom_as_lwline(lwgeom_from_gserialized(pglwg1));
 
-	if ( PG_NARGS() > 2 )
+	if (PG_NARGS() <= 2)
 	{
-		where = PG_GETARG_INT32(2);
+		uwhere = line->points->npoints;
 	}
 	else
 	{
-		where = line->points->npoints;
-	}
-
-	if ( where < 0 || where > (int32) line->points->npoints )
-	{
-		elog(ERROR, "Invalid offset");
-		PG_RETURN_NULL();
+		int32 where = PG_GETARG_INT32(2);
+		if (where == -1)
+		{
+			uwhere = line->points->npoints;
+		}
+		else if (where < 0 || where > (int32)line->points->npoints)
+		{
+			elog(ERROR, "%s: Invalid offset", __func__);
+			PG_RETURN_NULL();
+		}
+		else
+		{
+			uwhere = where;
+		}
 	}
 
 	point = lwgeom_as_lwpoint(lwgeom_from_gserialized(pglwg2));
 	linecopy = lwgeom_as_lwline(lwgeom_clone_deep(lwline_as_lwgeom(line)));
 	lwline_free(line);
 
-	if ( lwline_add_lwpoint(linecopy, point, (uint32_t) where) == LW_FAILURE )
+	if (lwline_add_lwpoint(linecopy, point, uwhere) == LW_FAILURE)
 	{
 		elog(ERROR, "Point insert failed");
 		PG_RETURN_NULL();
@@ -2221,7 +2227,6 @@ Datum LWGEOM_addpoint(PG_FUNCTION_ARGS)
 	lwpoint_free(point);
 
 	PG_RETURN_POINTER(result);
-
 }
 
 PG_FUNCTION_INFO_V1(LWGEOM_removepoint);
@@ -2305,6 +2310,12 @@ Datum LWGEOM_setpoint_linestring(PG_FUNCTION_ARGS)
 		elog(ERROR, "First argument must be a LINESTRING");
 		PG_RETURN_NULL();
 	}
+
+	if ( line->points->npoints < 1 )        {
+					elog(ERROR, "Line has no points");
+					PG_RETURN_NULL();
+	}
+
 	if(which < 0){
 		/* Use backward indexing for negative values */
 		which = which + line->points->npoints ;
@@ -2774,6 +2785,11 @@ Datum ST_RemoveRepeatedPoints(PG_FUNCTION_ARGS)
 
 	lwgeom_in = lwgeom_from_gserialized(g_in);
 	lwgeom_out = lwgeom_remove_repeated_points(lwgeom_in, tolerance);
+
+	/* COMPUTE_BBOX TAINTING */
+	if (lwgeom_in->bbox)
+		lwgeom_refresh_bbox(lwgeom_out);
+
 	g_out = geometry_serialize(lwgeom_out);
 
 	if ( lwgeom_out != lwgeom_in )
