@@ -437,7 +437,7 @@ ptarray_locate_between_m(POINTARRAY *ipa, double m0, double m1)
 {
 	POINTARRAYSET ret;
 	POINTARRAY *dpa=NULL;
-	int i;
+	uint32_t i;
 
 	ret.nptarrays=0;
 
@@ -527,7 +527,7 @@ lwpoint_locate_between_m(LWPOINT *lwpoint, double m0, double m1)
 	{
 		POSTGIS_DEBUG(3, " lwpoint... returning a clone of input");
 
-		return (LWGEOM *)lwpoint_clone(lwpoint);
+		return lwgeom_clone((LWGEOM *)lwpoint);
 	}
 	else
 	{
@@ -628,7 +628,7 @@ lwline_locate_between_m(LWLINE *lwline_in, double m0, double m1)
 static LWGEOM *
 lwcollection_locate_between_m(LWCOLLECTION *lwcoll, double m0, double m1)
 {
-	int i;
+	uint32_t i;
 	int ngeoms=0;
 	LWGEOM **geoms;
 
@@ -685,7 +685,7 @@ lwgeom_locate_between_m(LWGEOM *lwin, double m0, double m1)
 		return NULL;
 	}
 
-	lwpgerror("Unkonwn geometry type (%s:%d)", __FILE__, __LINE__);
+	lwpgerror("Unknown geometry type (%s:%d)", __FILE__, __LINE__);
 	return NULL;
 }
 
