@@ -10,11 +10,11 @@
 PG_FUNCTION_INFO_V1(postgis_libprotobuf_version);
 Datum postgis_libprotobuf_version(PG_FUNCTION_ARGS)
 {
-#ifndef HAVE_LIBPROTOBUF
+#ifndef HAVE_PROTOBUF_C_VERSION
 	PG_RETURN_NULL();
-#else /* HAVE_LIBPROTOBUF  */
+#else /* HAVE_PROTOBUF_C_VERSION  */
 	const char *ver = protobuf_c_version();
 	text *result = cstring_to_text(ver);
 	PG_RETURN_POINTER(result);
 #endif
-}
\ No newline at end of file
+}
