@@ -368,7 +368,7 @@ rt_raster_set_srid(rt_raster raster, int32_t srid) {
 	_rt_raster_geotransform_warn_offline_band(raster);
 }
 
-int
+uint16_t
 rt_raster_get_num_bands(rt_raster raster) {
 
 
@@ -1262,11 +1262,11 @@ rt_raster_compute_skewed_raster(
 			return NULL;
 		}
 
-		if (covers)
-			break;
-
-		raster->width++;
-		raster->height++;
+		if (!covers)
+		{
+			raster->width++;
+			raster->height++;
+		}
 	}
 	while (!covers);
 
@@ -1309,17 +1309,13 @@ rt_raster_compute_skewed_raster(
 				rt_raster_destroy(raster);
 				return NULL;
 			}
+		} while (covers);
 
-			if (!covers) {
-				if (i < 1)
-					raster->width++;
-				else
-					raster->height++;
+		if (i < 1)
+			raster->width++;
+		else
+			raster->height++;
 
-				break;
-			}
-		}
-		while (covers);
 	}
 
 	GEOSGeom_destroy(ngeom);
@@ -1336,7 +1332,7 @@ rt_raster_compute_skewed_raster(
  */
 int
 rt_raster_is_empty(rt_raster raster) {
-	return (NULL == raster || raster->height <= 0 || raster->width <= 0);
+	return (!raster || raster->height == 0 || raster->width == 0);
 }
 
 /**
@@ -1603,6 +1599,9 @@ rt_raster_to_gdal(
 	rt_raster raster, const char *srs,
 	char *format, char **options, uint64_t *gdalsize
 ) {
+	const char *cc;
+	const char *vio;
+
 	GDALDriverH src_drv = NULL;
 	int destroy_src_drv = 0;
 	GDALDatasetH src_ds = NULL;
@@ -1643,6 +1642,18 @@ rt_raster_to_gdal(
 	}
 	RASTER_DEBUG(3, "Output driver loaded");
 
+	/* CreateCopy support */
+	cc = GDALGetMetadataItem(rtn_drv, GDAL_DCAP_CREATECOPY, NULL);
+	/* VirtualIO support */
+	vio = GDALGetMetadataItem(rtn_drv, GDAL_DCAP_VIRTUALIO, NULL);
+
+	if (cc == NULL || vio == NULL) {
+		rterror("rt_raster_to_gdal: Output GDAL driver does not support CreateCopy and/or VirtualIO");
+		GDALClose(src_ds);
+		if (destroy_src_drv) GDALDestroyDriver(src_drv);
+		return 0;
+	}
+
 	/* convert GDAL MEM raster to output format */
 	RASTER_DEBUG(3, "Copying GDAL MEM raster to memory file in output format");
 	rtn_ds = GDALCreateCopy(
@@ -1696,14 +1707,15 @@ rt_raster_to_gdal(
  * Returns a set of available GDAL drivers
  *
  * @param drv_count : number of GDAL drivers available
- * @param cancc : if non-zero, filter drivers to only those
+ * @param can_write : if non-zero, filter drivers to only those
  *   with support for CreateCopy and VirtualIO
  *
  * @return set of "gdaldriver" values of available GDAL drivers
  */
 rt_gdaldriver
-rt_raster_gdal_drivers(uint32_t *drv_count, uint8_t cancc) {
-	const char *state;
+rt_raster_gdal_drivers(uint32_t *drv_count, uint8_t can_write) {
+	const char *cc;
+	const char *vio;
 	const char *txt;
 	int txt_len;
 	GDALDriverH *drv = NULL;
@@ -1730,19 +1742,28 @@ rt_raster_gdal_drivers(uint32_t *drv_count, uint8_t cancc) {
 #ifdef GDAL_DCAP_RASTER
 		/* Starting with GDAL 2.0, vector drivers can also be returned */
 		/* Only keep raster drivers */
-		state = GDALGetMetadataItem(drv, GDAL_DCAP_RASTER, NULL);
-		if (state == NULL || !EQUAL(state, "YES"))
+		const char *is_raster;
+		is_raster = GDALGetMetadataItem(drv, GDAL_DCAP_RASTER, NULL);
+		if (is_raster == NULL || !EQUAL(is_raster, "YES"))
 			continue;
 #endif
 
-		if (cancc) {
-			/* CreateCopy support */
-			state = GDALGetMetadataItem(drv, GDAL_DCAP_CREATECOPY, NULL);
-			if (state == NULL) continue;
+		/* CreateCopy support */
+		cc = GDALGetMetadataItem(drv, GDAL_DCAP_CREATECOPY, NULL);
 
-			/* VirtualIO support */
-			state = GDALGetMetadataItem(drv, GDAL_DCAP_VIRTUALIO, NULL);
-			if (state == NULL) continue;
+		/* VirtualIO support */
+		vio = GDALGetMetadataItem(drv, GDAL_DCAP_VIRTUALIO, NULL);
+
+		if (can_write && (cc == NULL || vio == NULL))
+			continue;
+
+		/* we can always read what GDAL can load */
+		rtn[j].can_read = 1;
+		/* we require CreateCopy and VirtualIO support to write to GDAL */
+		rtn[j].can_write = (cc != NULL && vio != NULL);
+
+		if (rtn[j].can_write) {
+			RASTER_DEBUGF(3, "driver %s (%d) supports CreateCopy() and VirtualIO()", txt, i);
 		}
 
 		/* index of driver */
@@ -1752,10 +1773,6 @@ rt_raster_gdal_drivers(uint32_t *drv_count, uint8_t cancc) {
 		txt = GDALGetDriverShortName(drv);
 		txt_len = strlen(txt);
 
-		if (cancc) {
-			RASTER_DEBUGF(3, "driver %s (%d) supports CreateCopy() and VirtualIO()", txt, i);
-		}
-
 		txt_len = (txt_len + 1) * sizeof(char);
 		rtn[j].short_name = (char *) rtalloc(txt_len);
 		memcpy(rtn[j].short_name, txt, txt_len);
@@ -1829,7 +1846,7 @@ rt_raster_to_gdal_mem(
 	int allocNodataValues = 0;
 
 	int i;
-	int numBands;
+	uint32_t numBands;
 	uint32_t width = 0;
 	uint32_t height = 0;
 	rt_band rtband = NULL;
@@ -2023,9 +2040,9 @@ rt_raster_to_gdal_mem(
 
 		/* PT_8BSI requires manual setting of pixels */
 		if (pt == PT_8BSI) {
-			int nXBlocks, nYBlocks;
+			uint32_t nXBlocks, nYBlocks;
 			int nXBlockSize, nYBlockSize;
-			int iXBlock, iYBlock;
+			uint32_t iXBlock, iYBlock;
 			int nXValid, nYValid;
 			int iX, iY;
 			int iXMax, iYMax;
@@ -2169,7 +2186,7 @@ rt_raster_from_gdal_dataset(GDALDatasetH ds) {
 	uint32_t width = 0;
 	uint32_t height = 0;
 	uint32_t numBands = 0;
-	int i = 0;
+	uint32_t i = 0;
 	char *authname = NULL;
 	char *authcode = NULL;
 
@@ -2185,11 +2202,11 @@ rt_raster_from_gdal_dataset(GDALDatasetH ds) {
 	int x;
 	int y;
 
-	int nXBlocks, nYBlocks;
+	uint32_t nXBlocks, nYBlocks;
 	int nXBlockSize, nYBlockSize;
-	int iXBlock, iYBlock;
-	int nXValid, nYValid;
-	int iY;
+	uint32_t iXBlock, iYBlock;
+	uint32_t nXValid, nYValid;
+	uint32_t iY;
 
 	uint8_t *values = NULL;
 	uint32_t valueslen = 0;
@@ -2503,7 +2520,7 @@ rt_raster_gdal_rasterize(
 	char **options
 ) {
 	rt_raster rast = NULL;
-	int i = 0;
+	uint32_t i = 0;
 	int err = 0;
 
 	_rti_rasterize_arg arg = NULL;
@@ -2702,129 +2719,29 @@ rt_raster_gdal_rasterize(
 		_dim[0] == 0 &&
 		_dim[1] == 0
 	) {
-		int result;
-		LWPOLY *epoly = NULL;
-		LWGEOM *lwgeom = NULL;
-		GEOSGeometry *egeom = NULL;
-		GEOSGeometry *geom = NULL;
-
-		RASTER_DEBUG(3, "Testing geometry is properly contained by extent");
-
-		/*
-			see if geometry is properly contained by extent
-			all parts of geometry lies within extent
-		*/
-
-		/* initialize GEOS */
-		initGEOS(rtinfo, lwgeom_geos_error);
-
-		/* convert envelope to geometry */
-		RASTER_DEBUG(4, "Converting envelope to geometry");
-		epoly = rt_util_envelope_to_lwpoly(extent);
-		if (epoly == NULL) {
-			rterror("rt_raster_gdal_rasterize: Could not create envelope's geometry to test if geometry is properly contained by extent");
-
-			OGR_G_DestroyGeometry(src_geom);
-			_rti_rasterize_arg_destroy(arg);
-			/* OGRCleanupAll(); */
-
-			return NULL;
-		}
-
-		egeom = (GEOSGeometry *) LWGEOM2GEOS(lwpoly_as_lwgeom(epoly), 0);
-		lwpoly_free(epoly);
-
-		/* convert WKB to geometry */
-		RASTER_DEBUG(4, "Converting WKB to geometry");
-		lwgeom = lwgeom_from_wkb(wkb, wkb_len, LW_PARSER_CHECK_NONE);
-		geom = (GEOSGeometry *) LWGEOM2GEOS(lwgeom, 0);
-		lwgeom_free(lwgeom);
-
-		result = GEOSRelatePattern(egeom, geom, "T**FF*FF*");
-		GEOSGeom_destroy(geom);
-		GEOSGeom_destroy(egeom);
-
-		if (result == 2) {
-			rterror("rt_raster_gdal_rasterize: Could not test if geometry is properly contained by extent for geometry within extent");
-
-			OGR_G_DestroyGeometry(src_geom);
-			_rti_rasterize_arg_destroy(arg);
-			/* OGRCleanupAll(); */
-
-			return NULL;
-		}
-
-		/* geometry NOT properly contained by extent */
-		if (!result) {
 
 #if POSTGIS_GDAL_VERSION > 18
 
-			/* check alignment flag: grid_xw */
-			if (
-				(NULL == ul_xw && NULL == ul_yw) &&
-				(NULL != grid_xw && NULL != grid_xw) &&
-				FLT_NEQ(*grid_xw, extent.MinX)
-			) {
-				/* do nothing */
-				RASTER_DEBUG(3, "Skipping extent adjustment on X-axis due to upcoming alignment");
-			}
-			else {
-				RASTER_DEBUG(3, "Adjusting extent for GDAL > 1.8 by half the scale on X-axis");
-				extent.MinX -= (_scale[0] / 2.);
-				extent.MaxX += (_scale[0] / 2.);
-			}
+		RASTER_DEBUG(3, "Adjusting extent for GDAL > 1.8 by half the scale on X-axis");
+		extent.MinX -= (_scale[0] / 2.);
+		extent.MaxX += (_scale[0] / 2.);
 
-			/* check alignment flag: grid_yw */
-			if (
-				(NULL == ul_xw && NULL == ul_yw) &&
-				(NULL != grid_xw && NULL != grid_xw) &&
-				FLT_NEQ(*grid_yw, extent.MaxY)
-			) {
-				/* do nothing */
-				RASTER_DEBUG(3, "Skipping extent adjustment on Y-axis due to upcoming alignment");
-			}
-			else {
-				RASTER_DEBUG(3, "Adjusting extent for GDAL > 1.8 by half the scale on Y-axis");
-				extent.MinY -= (_scale[1] / 2.);
-				extent.MaxY += (_scale[1] / 2.);
-			}
+		RASTER_DEBUG(3, "Adjusting extent for GDAL > 1.8 by half the scale on Y-axis");
+		extent.MinY -= (_scale[1] / 2.);
+		extent.MaxY += (_scale[1] / 2.);
 
 #else
 
-			/* check alignment flag: grid_xw */
-			if (
-				(NULL == ul_xw && NULL == ul_yw) &&
-				(NULL != grid_xw && NULL != grid_xw) &&
-				FLT_NEQ(*grid_xw, extent.MinX)
-			) {
-				/* do nothing */
-				RASTER_DEBUG(3, "Skipping extent adjustment on X-axis due to upcoming alignment");
-			}
-			else {
-				RASTER_DEBUG(3, "Adjusting extent for GDAL <= 1.8 by the scale on X-axis");
-				extent.MinX -= _scale[0];
-				extent.MaxX += _scale[0];
-			}
+		RASTER_DEBUG(3, "Adjusting extent for GDAL <= 1.8 by the scale on X-axis");
+		extent.MinX -= _scale[0];
+		extent.MaxX += _scale[0];
 
-
-			/* check alignment flag: grid_yw */
-			if (
-				(NULL == ul_xw && NULL == ul_yw) &&
-				(NULL != grid_xw && NULL != grid_xw) &&
-				FLT_NEQ(*grid_yw, extent.MaxY)
-			) {
-				/* do nothing */
-				RASTER_DEBUG(3, "Skipping extent adjustment on Y-axis due to upcoming alignment");
-			}
-			else {
-				RASTER_DEBUG(3, "Adjusting extent for GDAL <= 1.8 by the scale on Y-axis");
-				extent.MinY -= _scale[1];
-				extent.MaxY += _scale[1];
-			}
+		RASTER_DEBUG(3, "Adjusting extent for GDAL <= 1.8 by the scale on Y-axis");
+		extent.MinY -= _scale[1];
+		extent.MaxY += _scale[1];
 
 #endif
 
-		}
 
 		RASTER_DEBUGF(3, "Adjusted extent: %f, %f, %f, %f",
 			extent.MinX, extent.MinY, extent.MaxX, extent.MaxY);
@@ -3501,6 +3418,7 @@ rt_raster_from_two_rasters(
 			i = 0;
 			_offset[0][0] = 0.;
 			_offset[0][1] = 0.;
+			/* FALLTHROUGH */
 		case ET_LAST:
 		case ET_SECOND:
 			if (i < 0) {
