@@ -250,7 +250,7 @@ Datum RASTER_asGDALRaster(PG_FUNCTION_ARGS)
 
 				if (strlen(option)) {
 					options[j] = (char *) palloc(sizeof(char) * (strlen(option) + 1));
-					options[j] = option;
+					strcpy(options[j], option);
 					j++;
 				}
 			}
@@ -323,20 +323,15 @@ Datum RASTER_asGDALRaster(PG_FUNCTION_ARGS)
 	SET_VARSIZE(result, result_size);
 	memcpy(VARDATA(result), gdal, VARSIZE(result) - VARHDRSZ);
 
-	/* for test output
-	FILE *fh = NULL;
-	fh = fopen("/tmp/out.dat", "w");
-	fwrite(gdal, sizeof(uint8_t), gdal_size, fh);
-	fclose(fh);
-	*/
-
 	/* free gdal mem buffer */
-	if (gdal) CPLFree(gdal);
+	CPLFree(gdal);
 
 	POSTGIS_RT_DEBUG(3, "RASTER_asGDALRaster: Returning pointer to GDAL raster");
 	PG_RETURN_POINTER(result);
 }
 
+#define VALUES_LENGTH 6
+
 /**
  * Returns available GDAL drivers
  */
@@ -362,7 +357,7 @@ Datum RASTER_getGDALDrivers(PG_FUNCTION_ARGS)
 		/* switch to memory context appropriate for multiple function calls */
 		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
 
-		drv_set = rt_raster_gdal_drivers(&drv_count, 1);
+		drv_set = rt_raster_gdal_drivers(&drv_count, 0);
 		if (NULL == drv_set || !drv_count) {
 			elog(NOTICE, "No GDAL drivers found");
 			MemoryContextSwitchTo(oldcontext);
@@ -403,24 +398,27 @@ Datum RASTER_getGDALDrivers(PG_FUNCTION_ARGS)
 
 	/* do when there is more left to send */
 	if (call_cntr < max_calls) {
-		int values_length = 4;
-		Datum values[values_length];
-		bool nulls[values_length];
+		Datum values[VALUES_LENGTH];
+		bool nulls[VALUES_LENGTH];
 		HeapTuple tuple;
 		Datum result;
 
 		POSTGIS_RT_DEBUGF(3, "Result %d", call_cntr);
 
-		memset(nulls, FALSE, sizeof(bool) * values_length);
+		memset(nulls, FALSE, sizeof(bool) * VALUES_LENGTH);
 
 		values[0] = Int32GetDatum(drv_set2[call_cntr].idx);
 		values[1] = CStringGetTextDatum(drv_set2[call_cntr].short_name);
 		values[2] = CStringGetTextDatum(drv_set2[call_cntr].long_name);
-		values[3] = CStringGetTextDatum(drv_set2[call_cntr].create_options);
+		values[3] = BoolGetDatum(drv_set2[call_cntr].can_read);
+		values[4] = BoolGetDatum(drv_set2[call_cntr].can_write);
+		values[5] = CStringGetTextDatum(drv_set2[call_cntr].create_options);
 
 		POSTGIS_RT_DEBUGF(4, "Result %d, Index %d", call_cntr, drv_set2[call_cntr].idx);
 		POSTGIS_RT_DEBUGF(4, "Result %d, Short Name %s", call_cntr, drv_set2[call_cntr].short_name);
 		POSTGIS_RT_DEBUGF(4, "Result %d, Full Name %s", call_cntr, drv_set2[call_cntr].long_name);
+		POSTGIS_RT_DEBUGF(4, "Result %d, Can Read %s", call_cntr, drv_set2[call_cntr].can_read);
+		POSTGIS_RT_DEBUGF(4, "Result %d, Can Write %s", call_cntr, drv_set2[call_cntr].can_write);
 		POSTGIS_RT_DEBUGF(5, "Result %d, Create Options %s", call_cntr, drv_set2[call_cntr].create_options);
 
 		/* build a tuple */
@@ -645,7 +643,7 @@ Datum RASTER_GDALWarp(PG_FUNCTION_ARGS)
 
 		dst_srs = rtpg_getSR(dst_srid);
 		if (NULL == dst_srs) {
-			if (!no_srid) pfree(src_srs);
+			pfree(src_srs);
 			rt_raster_destroy(raster);
 			PG_FREE_IF_COPY(pgraster, 0);
 			elog(ERROR, "RASTER_GDALWarp: Target SRID (%d) is unknown", dst_srid);
