@@ -70,6 +70,7 @@ my $sql_file = $ARGV[0];
 my $module = 'postgis';
 my $soname = '';
 my $version_to = "";
+my $version_to_full = "";
 my $version_to_num = 0;
 my $version_from = $ARGV[1];
 my $version_from_num = 0;
@@ -108,10 +109,11 @@ close(INPUT);
 die "Unable to locate target new version number in $sql_file\n"
  	if( ! $version_to );
 
-if ( $version_to =~ /(\d+)\.(\d+)\..*/ )
+if ( $version_to =~ /(\d+)\.(\d+)\.(\d+)([^' ]*)/ )
 {
-	$version_to = $1 . "." . $2;
-	$version_to_num = 100 * $1 + $2;
+    $version_to_full = $1 . '.' . $2 . '.' . $3 . $4;
+    $version_to = $1 . "." . $2;
+    $version_to_num = 100 * $1 + $2;
 }
 else
 {
@@ -120,7 +122,7 @@ else
 
 print qq{
 --
--- UPGRADE SCRIPT TO PostGIS $version_to
+-- UPGRADE SCRIPT TO PostGIS $version_to_full
 --
 
 };
@@ -136,9 +138,9 @@ print "SET search_path TO $schema;\n" if $schema;
 #
 while(<DATA>)
 {
-	s/NEWVERSION/$version_to/g;
-  s/MODULE/$module/g;
-	print;
+    s/NEWVERSION/$version_to_full/g;
+    s/MODULE/$module/g;
+    print;
 }
 
 #
@@ -469,42 +471,85 @@ __END__
 
 CREATE OR REPLACE FUNCTION postgis_major_version_check()
 RETURNS text
-AS '
+AS $BODY$
 DECLARE
-	old_scripts text;
-	new_scripts text;
-	old_maj text;
-	new_maj text;
+    old_scripts text;
+    new_scripts text;
+    old_ver_int int[];
+    new_ver_int int[];
+    old_maj text;
+    new_maj text;
 BEGIN
-	--
-	-- This uses postgis_lib_version() rather then
-	-- MODULE_scripts_installed() as in 1.0 because
-	-- in the 1.0 => 1.1 transition that would result
-	-- in an impossible upgrade:
-	--
-	--   from 0.3.0 to 1.1.0
-	--
-	-- Next releases will still be ok as
-	-- postgis_lib_version() and MODULE_scripts_installed()
-	-- would both return actual PostGIS release number.
-	--
-	BEGIN
-		SELECT into old_scripts MODULE_lib_version();
-	EXCEPTION WHEN OTHERS THEN
-		RAISE DEBUG ''Got %'', SQLERRM;
-		SELECT into old_scripts MODULE_scripts_installed();
-	END;
-	SELECT into new_scripts ''NEWVERSION'';
-	SELECT into old_maj substring(old_scripts from 1 for 2);
-	SELECT into new_maj substring(new_scripts from 1 for 2);
+    --
+    -- This uses postgis_lib_version() rather then
+    -- MODULE_scripts_installed() as in 1.0 because
+    -- in the 1.0 => 1.1 transition that would result
+    -- in an impossible upgrade:
+    --
+    --   from 0.3.0 to 1.1.0
+    --
+    -- Next releases will still be ok as
+    -- postgis_lib_version() and MODULE_scripts_installed()
+    -- would both return actual PostGIS release number.
+    --
+    BEGIN
+        SELECT into old_scripts MODULE_lib_version();
+    EXCEPTION WHEN OTHERS THEN
+        RAISE DEBUG 'Got %', SQLERRM;
+        SELECT into old_scripts MODULE_scripts_installed();
+    END;
+    SELECT into new_scripts 'NEWVERSION';
 
-	IF old_maj != new_maj THEN
-		RAISE EXCEPTION ''Upgrade of MODULE from version % to version % requires a dump/reload. See PostGIS manual for instructions'', old_scripts, new_scripts;
-	ELSE
-		RETURN ''Scripts versions checked for upgrade: ok'';
-	END IF;
-END
-'
+    BEGIN
+        new_ver_int := pg_catalog.string_to_array(
+            pg_catalog.regexp_replace(
+                new_scripts,
+                '[^\d.].*',
+                ''
+            ),
+            '.'
+        )::int[];
+    EXCEPTION WHEN OTHERS THEN
+        RAISE EXCEPTION 'Cannot parse new version % into integers', new_scripts;
+    END;
+
+    BEGIN
+        old_ver_int := pg_catalog.string_to_array(
+            pg_catalog.regexp_replace(
+                old_scripts,
+                '[^\d.].*',
+                ''
+            ),
+            '.'
+        )::int[];
+    EXCEPTION WHEN OTHERS THEN
+        RAISE EXCEPTION 'Cannot parse old version % into integers', old_scripts;
+    END;
+
+    -- Guard against downgrade
+    IF new_ver_int < old_ver_int
+    THEN
+        RAISE EXCEPTION 'Downgrade of MODULE from version % to version % is forbidden', old_scripts, new_scripts;
+    END IF;
+
+
+    -- Check for hard-upgrade being required
+    SELECT into old_maj pg_catalog.substring(old_scripts,1, 1);
+    SELECT into new_maj pg_catalog.substring(new_scripts,1, 1);
+
+    -- 2.x to 3.x was upgrade-compatible, see
+    -- https://trac.osgeo.org/postgis/ticket/4170#comment:1
+    IF new_maj = '3' AND old_maj = '2' THEN
+        old_maj = '3'; -- let's pretend old major = new major
+    END IF;
+
+    IF old_maj != new_maj THEN
+        RAISE EXCEPTION 'Upgrade of MODULE from version % to version % requires a dump/reload. See PostGIS manual for instructions', old_scripts, new_scripts;
+    END IF;
+
+    RETURN 'Scripts versions checked for upgrade: ok';
+END;
+$BODY$
 LANGUAGE 'plpgsql';
 
 SELECT postgis_major_version_check();
@@ -517,11 +562,11 @@ CREATE TEMPORARY TABLE _postgis_upgrade_info AS WITH versions AS (
 ) SELECT
   upgraded as scripts_upgraded,
   installed as scripts_installed,
-  substring(upgraded from '([0-9]*)\.')::int * 100 +
-  substring(upgraded from '[0-9]*\.([0-9]*)\.')::int
+  pg_catalog.substring(upgraded,'([0-9]*)\.')::int * 100 +
+  pg_catalog.substring(upgraded, '[0-9]*\.([0-9]*)\.')::int
     as version_to_num,
-  substring(installed from '([0-9]*)\.')::int * 100 +
-  substring(installed from '[0-9]*\.([0-9]*)\.')::int
+  pg_catalog.substring(installed, '([0-9]*)\.')::int * 100 +
+  pg_catalog.substring(installed, '[0-9]*\.([0-9]*)\.')::int
     as version_from_num,
   installed ~ 'dev|alpha|beta'
     as version_from_isdev
